import { ComponentFixture, fakeAsync, TestBed } from '@angular/core/testing';
import { By } from '@angular/platform-browser';
import { of } from 'rxjs';
import { DataTableColumnFormatType } from '../data-table.model';
import { getNestedAttribute } from '../data-table.utils';
import { CustomDataTableCellComponent } from './custom-data-table-cell.component';

describe('CustomDataTableCellComponent', () => {
  let component: CustomDataTableCellComponent;
  let fixture: ComponentFixture<CustomDataTableCellComponent>;

  const mockStatusColumn = {
    format: {
      type: DataTableColumnFormatType.statusToggle,
      action: () => {
        return of({ responseStatus: 'SUCCESS' });
      },
    },
  };
  const mockNestedObjectColumn = {
    format: { type: DataTableColumnFormatType.nestedAttribute },
    field: 'test.mock.name',
  };
  const mockRecord = { id: 42, test: { mock: { name: 'testMockValue' } }, status: false };
  const mockMouseEvent = { preventDefault: () => {} };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [CustomDataTableCellComponent],
    }).compileComponents();
  });

  describe('StatusToggle Column', () => {
    beforeEach(() => {
      fixture = TestBed.createComponent(CustomDataTableCellComponent);
      component = fixture.componentInstance;
      component.column = mockStatusColumn as any;
      component.record = mockRecord;
      fixture.detectChanges();
    });

    it('should create component', () => {
      expect(component).toBeTruthy();
    });
    it('should toggle status', fakeAsync(() => {
      spyOn<any>(component.column.format, 'action').and.callThrough();
      spyOn<any>(mockMouseEvent, 'preventDefault');

      component.onStatusToggleClick(mockMouseEvent as any, mockStatusColumn as any, mockRecord);

      expect(component.column.format?.action).toHaveBeenCalledOnceWith(mockRecord.id, {
        ...mockRecord,
        status: true,
      });
      expect(component.record.status).toBeTrue();
      expect(mockMouseEvent.preventDefault).not.toHaveBeenCalled();
    }));
  });
  describe('NestedObject Column', () => {
    beforeEach(() => {
      fixture = TestBed.createComponent(CustomDataTableCellComponent);
      component = fixture.componentInstance;
      component.column = mockNestedObjectColumn as any;
      component.record = mockRecord;
      fixture.detectChanges();
    });

    it('should create component', () => {
      expect(component).toBeTruthy();
    });
    it('should display nested value', fakeAsync(() => {
      const cellDisplayValueSpan = fixture.debugElement.query(By.css('span'));
      expect(cellDisplayValueSpan).toBeDefined();
      expect(cellDisplayValueSpan.nativeNode?.innerText).toEqual(mockRecord.test.mock.name);
    }));
    it('should display empty value if nested attribute does not exist', () => {
      const mockFailure = getNestedAttribute(
        { ...mockRecord, test: null },
        mockNestedObjectColumn.field
      );
      expect(mockFailure).toBeUndefined();
    });
  });
});
