import { UmbrellaLibraryService } from '../../../umbrella-library.service';
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input } from '@angular/core';
import { finalize } from 'rxjs/operators';

import { DataTableColumn, DataTableColumnFormatType } from '../data-table.model';
import { getNestedAttribute } from '../data-table.utils';

@Component({
  selector: 'x-custom-data-table-cell',
  templateUrl: './custom-data-table-cell.component.html',
  styleUrls: ['./custom-data-table-cell.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CustomDataTableCellComponent {
  @Input() column!: DataTableColumn;
  @Input() record: any;

  readonly dataTableColumnFormatType = DataTableColumnFormatType;
  readonly getNestedAttribute = getNestedAttribute;

  loadingStatusToggleRecordId!: number;

  constructor(private cdr: ChangeDetectorRef, private library: UmbrellaLibraryService) {}

  onStatusToggleClick(event: MouseEvent, column: DataTableColumn, record: any) {
    if (column.format?.type === DataTableColumnFormatType.statusToggle) {
      this.loadingStatusToggleRecordId = record.id;
      column.format
        .action(record.id, { ...record, status: !record.status })
        .pipe(
          finalize(() => {
            this.loadingStatusToggleRecordId = -1;
            this.cdr.markForCheck();
          })
        )
        .subscribe((response: any) => {
          if (!this.library.isSuccessfulResponse(response)) {
            event.preventDefault();
          } else {
            record.status = !record.status;
          }
        });
    }
  }
}
