/**
 *
 * @param object a table data record
 * @param nestedAttributePath the path of a nested attribute to display
 * @returns the value of a sub object located at the nested path of the input object
 * @example
 * ```ts
 * const exObj = {fleet: {vehicleGroup: {id: 1, name: 'v group 1'}}};
 * const nestedValue = getNestedAttribute(exObj, 'fleet.vehicleGroup.name');
 * console.log(nestedValue);
 * // v group 1
 * ```
 */
export function getNestedAttribute(object: any, nestedAttributePath: string) {
  const nestedAttributes = nestedAttributePath.split('.');
  let nestedAttributeValue = object;
  for (let nestedAttribute of nestedAttributes) {
    if (nestedAttributeValue[nestedAttribute]) {
      nestedAttributeValue = nestedAttributeValue[nestedAttribute];
    } else {
      nestedAttributeValue = undefined;
      break;
    }
  }
  return nestedAttributeValue;
}
